(($) => {

    function disableScreen() {
        var div = document.createElement("div");
        div.className += "overlay";
        div.style.backgroundColor = "#EFEFEF";
        div.style.position = "fixed";
        div.style.width = "100%";
        div.style.height = "100%";
        div.style.zIndex = "999999999999999";
        div.style.top = "0px";
        div.style.left = "0px";
        div.style.opacity = ".5";
        document.body.appendChild(div);
    }

    function enableScreen() {
        if (document.querySelector(".overlay")) {
            document.querySelector(".overlay").remove();
        }
    }

    function swalPopup(message, type, html = null) {
        enableScreen();
        return Swal.fire({
            title: message,
            html,
            icon: type,
            didOpen: () => {
                Swal.hideLoading();
            }
        });
    }
    function infoPopup(message, html = null) {
        return swalPopup(message, 'info', html);
    }
    
    function errorPopup(message, html = null) {
        return swalPopup(message, 'error', html);
    }
    
    function successPopup(message, html = null) {
        return swalPopup(message, 'success', html);
    }
    
    function waitingPopup(title, html = null) {
        Swal.fire({
            title,
            html,
            allowOutsideClick: false,
            didOpen: () => {
                Swal.showLoading();
            }
        });
    }

    Walogin.infoPopup = infoPopup;
    Walogin.errorPopup = errorPopup;
    Walogin.successPopup = successPopup;
    Walogin.waitingPopup = waitingPopup;
    Walogin.disableScreen = disableScreen;
    Walogin.enableScreen = enableScreen;

    function errorMessages(error) {
        if (error == 'closed-web3modal') return;

        if (typeof error === 'object') {
            if (String(error.message).includes('Already processing')) {
                return infoPopup(Walogin.lang.alreadyProcessing);
            }
        }

        return infoPopup(Walogin.lang.requestRejected);
    }

    const w3m = Walogin.w3m = new Web3Modal({
        projectId: Walogin.wcProjectId
    });

    // Hooks
    Walogin.hooks = [];

    Walogin.addHook = (hook, callback, priority = 10) => {
        if (typeof callback === 'function') {
            if (typeof Walogin.hooks[hook] === 'undefined') {
                Walogin.hooks[hook] = [];
            }
            Walogin.hooks[hook].push({priority, callback});
            Walogin.hooks[hook].sort((a, b) => a.priority - b.priority);
        } else {
            throw new Error('Callback must be a function');
        }

        return true;
    };

    Walogin.callHook = (hook, ...params) => {
        if (typeof Walogin.hooks[hook] !== 'undefined') {
            Walogin.hooks[hook].forEach((hook) => {
                hook.callback(...params);
            });
        } else {
            throw new Error('Not found hook');
        }

        return true;
    };

    Walogin.removeHook = (hook) => {
        if (typeof Walogin.hooks[hook] !== 'undefined') {
            delete Walogin.hooks[hook];
        } else {
            throw new Error('Not found hook');
        }

        return true;
    };

    function updateCookie(key, val) {
        let cookie = $.cookie('wl-connected') || {};
        cookie = typeof cookie !== 'object' ? JSON.parse(cookie) : cookie;
        cookie[key] = val;
        $.cookie('wl-connected', JSON.stringify(cookie), {path: '/'});
    }

    function getCookie(key = null) {
        let cookie = $.cookie('wl-connected') || {};
        cookie = typeof cookie !== 'object' ? JSON.parse(cookie) : cookie;
        if (!key) return cookie;
        return cookie[key] || null;
    }

    const afterConnectProcess = async (connectedAccount, type) => {
        if (type == 'only-connect-wallet') {
            Swal.close();
            $(".walogin-connect").addClass('hidden');
        } else if (type == 'matching') {
            matchingProcess(connectedAccount);
        } else if (type == 'change-address') {
            addressChangeProcess(connectedAccount);
        } else if (type == 'remove-matching') {
            removeMatchingProcess(connectedAccount);
        } else if (type == 'login-register') {
            loginProcess(connectedAccount);
        } else {
            Swal.close();
            Walogin.callHook('connectAfter', connectedAccount, type);
        }
    }

    let w3mConnected = false;
    const w3mConnect = async (type) => {
        if (!w3mConnected) {
            await w3m.removeOldConnection();
        }
        await new Promise((r) => setTimeout(r, 100));
        if (w3m.connectedAccount) {
            updateCookie('address', w3m.connectedAccount);
            return afterConnectProcess(w3m.connectedAccount, type);
        } else {
            return w3m.connect()
            .then(async (connectedAccount) => {
                w3mConnected = true;
                updateCookie('address', connectedAccount);
                afterConnectProcess(connectedAccount, type);
            })
            .catch(error => {
                errorMessages(error);
            });
        }
    };

    // events

    $(document).on('click', ".walogin-connect", async function(e) {
        e.preventDefault();
        w3mConnect($(this).data("type"));
    });
    
    $(document).on('click', ".walogin-try-again-login", async function(e) {
        e.preventDefault();
        w3mConnect('login-register');
    });

    // sign

    function getSignMessage(address) {
        return new Promise((resolve, reject) => {
            $.ajax({
                method: 'POST',
                url: Walogin.apiUrl + '/get-sign-message',
                data: {
                    address
                },
                success(response) {
                    if (response.success) {
                        resolve(response.message);
                    } else {
                        reject(response);
                    }
                },
                error(error) {
                    errorMessages(error)
                }
            });
        });
    }

    function signProcess(address) {
        return new Promise((resolve, reject) => {
            getSignMessage(address)
            .then(message => {
                w3m.personalSign(message)
                .then(signature => {
                    updateCookie('signature', signature);
                    resolve(signature);
                })
                .catch(error => {
                    if (error == 'signature-request-denied') {
                        infoPopup(Walogin.lang.signatureRequestDenied);
                    } else {
                        errorMessages(error)
                    }
                    reject(error);
                });
            })
            .catch(error => {
                errorMessages(error)
                reject(error);
            });
        });
    }

    // login register
    function login(address, signature) {
        let redirectTo = (new URLSearchParams(window.location.search)).get('redirect_to');
        redirectTo = Walogin.redirectUrl == 'same-page' ? window.location.href : redirectTo;
        return new Promise((resolve, reject) => {
            $.ajax({
                method: 'POST',
                url: Walogin.apiUrl + '/login',
                data: {
                    address,
                    signature,
                    redirectTo
                },
                beforeSend() {
                    waitingPopup(Walogin.lang.loginPleaseWait);
                },
                success(response) {
                    if (response.success) {
                        try {
                            Walogin.callHook('loginAfter', {
                                counnectedAccount: w3m.connectedAccount, 
                                response
                            });
                        } catch (error) {
                            successPopup(response.message).then(() => {
                                disableScreen();
                                location.href = response.data.redirectTo;
                            });
                        }
                        resolve(true);
                    } else {
                        reject(response);
                    }
                },
                error() {
                    reject(Walogin.lang.somethingWentWrong);
                }
            });
        });
    }

    function loginProcess(address) {
        waitingPopup(Walogin.lang.youMustSignToEnter, `
            <button type="button" class="swal2-confirm swal2-styled swal2-default-outline walogin-try-again-login" aria-label="" style="display: inline-block; background-color: rgb(48, 133, 214);">${Walogin.lang.tryAgain}</button>
        `);
        signProcess(address)
        .then(signature => {
            login(address, signature)
            .catch(error => {
                if (error.errorCode == 'MEMBERSHIP_NOT_FOUND') {
                    infoPopup(error.message);
                } else if (error.errorCode == 'REGISTER') {
                    registerProcess(address, signature);
                }
            });
        });
    }

    function register(address, signature, username, email) {
        let redirectTo = (new URLSearchParams(window.location.search)).get('redirect_to') || null;
        return new Promise((resolve, reject) => {
            $.ajax({
                method: 'POST',
                url: Walogin.apiUrl + '/register',
                data: {
                    address,
                    signature,
                    redirectTo,
                    username,
                    email
                },
                success(response) {
                    if (response.success) {
                        resolve(response);
                    } else {
                        reject(response);
                    }
                },
                error() {
                    reject(Walogin.lang.somethingWentWrong);
                }
            });
        });
    }

    function registerProcess(address, signature) {
        Swal.fire({
            title: Walogin.lang.registerQuestion,
            icon: 'info',
            showCancelButton: true,
            confirmButtonText: Walogin.lang.register
        }).then(async (result) => {
            if (result.isConfirmed) {
                Swal.fire({
                    title: Walogin.lang.pleaseFillOut,
                    html:
                        '<input type="text" id="wl-username" class="swal2-input" placeholder="'+Walogin.lang.username+'" required>' +
                        '<input type="email" id="wl-email" class="swal2-input" placeholder="'+Walogin.lang.email+'" required>',
                    focusConfirm: false,
                    showCancelButton: true,
                    confirmButtonText: Walogin.lang.register,
                    preConfirm: async () => {
                        
                        let email = document.getElementById('wl-email').value;
                        let username = document.getElementById('wl-username').value;

                        if (!email || !username) {
                            return Swal.showValidationMessage(Walogin.lang.pleaseFillError);
                        }

                        return await register(address, signature, username, email)
                        .catch(error => {
                            Swal.showValidationMessage(error.message);
                        });
                    }
                }).then(async (result) => {
                    if (result.isConfirmed) {
                        try {
                            Walogin.callHook('registerAfter', {
                                counnectedAccount: w3m.connectedAccount, 
                                response: result.value
                            });
                        } catch (error) {
                            successPopup(result.value.message).then(() => {
                                disableScreen();
                                location.href = result.value.data.redirectTo;
                            });
                        }
                    } else {
                        enableScreen();
                    }
                });
            } else {
                enableScreen();
            }
        });

    }

    // Matching

    function matchingControl(address) {
        return new Promise((resolve, reject) => {
            $.ajax({
                method: 'POST',
                url: Walogin.apiUrl + '/matching-control',
                data: {
                    address
                },
                beforeSend() {
                    waitingPopup(Walogin.lang.pleaseWait);
                },
                success(response) {
                    if (response.success) {
                        resolve(response);
                    } else {
                        reject(response);
                    }
                },
                error() {
                    reject(Walogin.lang.somethingWentWrong);
                }
            });
        });
    }

    function removeMatching(address, signature) {
        return new Promise((resolve, reject) => {
            $.ajax({
                method: 'POST',
                url: Walogin.apiUrl + '/remove-matching',
                data: {
                    address,
                    signature
                },
                beforeSend() {
                    waitingPopup(Walogin.lang.pleaseWait);
                },
                success(response) {
                    if (response.success) {
                        resolve(response.message);
                    } else {
                        reject(response);
                    }
                },
                error() {
                    reject(Walogin.lang.somethingWentWrong);
                }
            });
        });
    }

    function addressMatch(address, signature) {
        return new Promise((resolve, reject) => {
            $.ajax({
                method: 'POST',
                url: Walogin.apiUrl + '/address-match',
                data: {
                    address,
                    signature
                },
                beforeSend() {
                    waitingPopup(Walogin.lang.pleaseWait);
                },
                success(response) {
                    if (response.success) {
                        resolve(response.message);
                    } else {
                        reject(response);
                    }
                },
                error() {
                    reject(Walogin.lang.somethingWentWrong);
                }
            });
        });
    }

    function alreadyMatching(error) {
        Swal.fire({
            title: error.message + "<br><br>" + error.data.address,
            showConfirmButton: false,
            showDenyButton: true,
            showCancelButton: true,
            cancelButtonText: Walogin.lang.cancel,
            denyButtonText: Walogin.lang.removeMatching,
        }).then((result) => {
            if (result.isDenied) {
                removeMatchingProcess(error.data.address);
            } else {
                enableScreen();
            }
        });
    }

    function matchingProcess(address) {
        matchingControl(address)
        .then(() => {
            signProcess(address)
            .then(signature => {
                addressMatch(address, signature)
                .then(message => {
                    successPopup(message).then(() => {
                        disableScreen();
                        window.location.reload();
                    });
                })
                .catch(error => {
                    infoPopup(error.message);
                });
            });
        })
        .catch(error => {
            if (error.errorCode == "ALREADY_MATCHING") {
                alreadyMatching(error);
            } else {
                errorPopup(error.message);
            }
        });
    }
    
    function removeMatchingProcess(address) {
        Swal.fire({
            title: Walogin.lang.removeMatchingQuestion,
            text: Walogin.lang.matchAgain,
            icon: 'info',
            showCancelButton: true,
            confirmButtonText: Walogin.lang.removeMatchingConfirm
        }).then((result) => {
            if (result.isConfirmed) {
                waitingPopup(Walogin.lang.youMustSignToRemove);
                signProcess(address)
                .then(signature => {
                    removeMatching(address, signature)
                    .then((message) => {
                        successPopup(message).then(() => {
                            disableScreen();
                            window.location.reload();
                        });
                    })
                    .catch(error => {
                        errorPopup(error.message);
                    });
                });
            } else {
                enableScreen();
            }
        });
    }

    // Address change

    function addressChange(address, signature) {
        return new Promise((resolve, reject) => {
            $.ajax({
                method: 'POST',
                url: Walogin.apiUrl + '/address-change',
                data: {
                    address,
                    signature
                },
                success(response) {
                    if (response.success) {
                        resolve(response.message);
                    } else {
                        reject(response);
                    }
                },
                error() {
                    reject(Walogin.lang.somethingWentWrong);
                }
            });
        });
    }

    function addressChangeProcess(address) {
        matchingControl(address)
        .catch(error => {
            if (error.errorCode == "ADDRESS_CHANGE") {
                let msg = error.message + "<br><br>" + 
                Walogin.lang.old + ": " + error.data.oldAddress + "<br><br>" + 
                Walogin.lang.new + ": " + error.data.newAddress;
                Swal.fire({
                    title: msg,
                    text: Walogin.lang.changeAgain,
                    icon: 'warning',
                    showCancelButton: true,
                    confirmButtonColor: '#3085d6',
                    confirmButtonText: Walogin.lang.changeAddressConfirm
                }).then((result) => {
                    if (result.isConfirmed) {
                        waitingPopup(Walogin.lang.youMustSignToChange);
                        signProcess(address)
                        .then(signature => {
                            addressChange(address, signature)
                            .then((message) => {
                                $(".matching-address").text(address);
                                successPopup(message);
                            })
                            .catch(error => {
                                errorPopup(error.message);
                            });
                        });
                    } else {
                        enableScreen();
                    }
                });
            } else if (error.errorCode == "ALREADY_MATCHING") {
                alreadyMatching(error);
            } else {
                errorPopup(error.message);
            }
        });
    }
})(jQuery);